/*
 * Copyright © 2010, Marshall Pierce
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the <organization> nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.mpierce.jfixture;

import net.jcip.annotations.Immutable;

import java.util.Iterator;

/**
 * Utility methods for interacting with a ContextStack.
 *
 * @author Marshall Pierce <marshall@mpierce.org>
 */
@Immutable
final class ContextUtil {

    private ContextUtil() {
        // not instantiable
    }

    /**
     * Generate an exception message from the supplied context and message
     *
     * @param context the context to include in the exception
     * @param message exception message
     *
     * @return the exception
     */
    static FixtureException generateContextException(ContextStack context, String message) {
        return generateContextException(context, message, null);
    }

    /**
     * Generate an exception message from the supplied context, message and cause
     *
     * @param context context stack
     * @param message message to use in the exception
     * @param cause   an exception that caused the failure
     *
     * @return the exception
     */
    static FixtureException generateContextException(ContextStack context, String message, Throwable cause) {
        StringBuilder buf = new StringBuilder();

        Iterator<String> contextIter = context.iterator();

        buf.append("context: [");
        while (contextIter.hasNext()) {
            buf.append(contextIter.next());

            if (contextIter.hasNext()) {
                buf.append(", ");
            }
        }

        buf.append("] ");
        buf.append(message);

        return new FixtureException(buf.toString(), cause);
    }
}
